@@ -23,6 +23,8 @@
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
+#include "vrhomboid.h"
+#include "track.h"
 #include "bitmap.h"
 
 
@@ -30,7 +32,7 @@ namespace {
 
 char pbm_getrawbyte( FILE * f ) throw( Bitmap::Error )
   {
-  int ch = std::getc( f );
+  int ch = std::fgetc( f );
 
   if( ch == EOF )
     throw Bitmap::Error( "end-of-file reading pbm file.\n" );
@@ -79,6 +81,21 @@ bool pbm_getbit( FILE * f ) throw( Bitmap::Error )
   throw Bitmap::Error( "junk in pbm file where bits should be.\n" );
   }
 
+
+void mirror_left_right( std::vector< std::vector< bool > > data ) throw()
+  {
+  int height = data.size();
+  for( int row = 0; row < height; ++row )
+    reverse( data[row].begin(), data[row].end() );
+  }
+
+
+void mirror_top_bottom( std::vector< std::vector< bool > > data ) throw()
+  {
+  for( int u = 0, d = data.size() - 1; u < d; ++u, --d )
+    swap( data[u], data[d] );
+  }
+
 } // end namespace
 
 
@@ -213,11 +230,15 @@ bool Bitmap::find_rows_and_columns() throw()
         }
     if( b - t >= gapmin )
       {
-      // cut by a minimum
-      for( int i = t; i <= b; ++i ) if( v_outline[i] < v_outline[t] ) t = i;
-      for( int i = b; i >= t; --i ) if( v_outline[i] < v_outline[b] ) b = i;
-      Rectangle r1( r.left(), r.top() + b, r.right(), r.bottom() );
-      r.bottom( r.top() + t );
+      // cut by a minimum near the center
+      int mid = ( b + t ) / 2, half = ( b - t ) / 2, imin = mid;
+      for( int i = 0; i <= half && v_outline[imin] > 0; ++i )
+        {
+        if( v_outline[mid+i] < v_outline[imin] ) imin = mid + i;
+        if( v_outline[mid-i] < v_outline[imin] ) imin = mid - i;
+        }
+      Rectangle r1( r.left(), r.top() + imin, r.right(), r.bottom() );
+      r.bottom( r.top() + imin );
       _rectangle_vector.insert( _rectangle_vector.begin() + rindex + 1, r1 );
       if( find_columns( rindex ) == 0 ) ++rindex;
       split = true;
@@ -258,20 +279,6 @@ void Bitmap::mirror_diagonal() throw()
   }
 
 
-void Bitmap::mirror_left_right() throw()
-  {
-  for( int row = 0; row < _height; ++row )
-    reverse( data[row].begin(), data[row].end() );
-  }
-
-
-void Bitmap::mirror_top_bottom() throw()
-  {
-  for( int u = 0, d = _height - 1; u < d; ++u, --d )
-    swap( data[u], data[d] );
-  }
-
-
 void Bitmap::floodfill4( const Bitmap & source, Bitmap & dest,
                          int row, int col ) throw()
   {
@@ -357,8 +364,7 @@ Bitmap::Bitmap( const Bitmap & source, const Rectangle & r ) throw()
       r.right() >= source._width || r.bottom() >= source._height )
     Ocrad::internal_error( "bad parameter building a Bitmap from another one" );
 
-  _width = r.right() - r.left() + 1;
-  _height = r.bottom() - r.top() + 1;
+  _height = r.height(); _width = r.width();
 
   data.resize( _height );
   for( int row = 0; row < _height; ++row )
@@ -387,8 +393,8 @@ Bitmap::Bitmap( const Bitmap & source, int scale, int th ) throw()
     std::fprintf( stderr, "automatic threshold = %d\n", th );
     }
 
-  _width = source._width / scale;
   _height = source._height / scale;
+  _width = source._width / scale;
 
   data.resize( _height );
   for( int row = 0; row < _height; ++row )
@@ -410,7 +416,7 @@ Bitmap::Bitmap( const Bitmap & source, int scale, int th ) throw()
 
 Bitmap::Bitmap( const Bitmap & source, Type t ) throw( Error )
   {
-  _width = source._width; _height = source._height;
+  _height = source._height; _width = source._width;
 
   data.resize( _height );
   for( int row = 0; row < _height; ++row ) data[row].resize( _width );
@@ -443,12 +449,63 @@ Bitmap::Bitmap( const Bitmap & source, Type t ) throw( Error )
   }
 
 
+Bitmap::Bitmap( const std::vector< std::vector< int > > & matrix ) throw()
+  {
+  _height = matrix.size();
+  if( _height == 0 || ( _width = matrix[0].size() ) == 0 )
+    Ocrad::internal_error( "bad size building a Bitmap from an int matrix" );
+
+  data.resize( _height );
+  for( int row = 0; row < _height; ++row )
+    {
+    data[row].resize( _width );
+    const std::vector< int > & v1 = matrix[row];
+    std::vector< bool > & v2 = data[row];
+    for( int col = 0; col < _width; ++col ) v2[col] = ( v1[col] > 0 );
+    }
+  }
+
+
+void Bitmap::draw_rectangle( const Rectangle & re ) throw()
+  {
+  int l = std::max( 0, re.left() );
+  int t = std::max( 0, re.top() );
+  int r = std::min( _width - 1, re.right() );
+  int b = std::min( _height - 1, re.bottom() );
+  if( l == re.left() ) for( int row = t; row <= b; ++row ) data[row][l] = true;
+  if( t == re.top() ) for( int col = l; col <= r; ++col ) data[t][col] = true;
+  if( r == re.right() ) for( int row = t; row <= b; ++row ) data[row][r] = true;
+  if( b == re.bottom() ) for( int col = l; col <= r; ++col ) data[b][col] = true;
+  }
+
+
+void Bitmap::draw_track( const Track & tr ) throw()
+  {
+  int l = std::max( 0, tr.left() );
+  int r = std::min( _width - 1, tr.right() );
+  if( l == r ) return;
+  if( l == tr.left() )
+    for( int row = tr.top(l); row <= tr.bottom(l); ++row )
+      if( row >= 0 && row < _height ) data[row][l] = true;
+  if( r == tr.right() )
+    for( int row = tr.top(r); row <= tr.bottom(r); ++row )
+      if( row >= 0 && row < _height ) data[row][r] = true;
+  for( int col = l; col <= r; ++col )
+    {
+    int row = tr.top( col );
+    if( row >= 0 && row < _height ) data[row][col] = true;
+    row = tr.bottom( col );
+    if( row >= 0 && row < _height ) data[row][col] = true;
+    }
+  }
+
+
 int Bitmap::analyse_layout( int layout_level ) throw()
   {
   if( _rectangle_vector.size() != 0 )
     Ocrad::internal_error( "analyse_layout called twice" );
 
-  if( layout_level > 0 && _width > 200 && _height > 200 )
+  if( layout_level > 0 && layout_level <= 2 && _width > 200 && _height > 200 )
     {
     Bitmap reduced( *this, 10, 9 );
     Rectangle r( 0, 0, reduced._width - 1, reduced._height - 1 );
@@ -519,17 +576,18 @@ void Bitmap::transform( const Transformation & t ) throw()
   {
   switch( t.type() )
     {
-    case Transformation::none: break;
-    case Transformation::rotate90:  mirror_diagonal(); mirror_top_bottom();
+    case Transformation::none:
       break;
-    case Transformation::rotate180: mirror_left_right(); mirror_top_bottom();
-      break;
-    case Transformation::rotate270: mirror_diagonal(); mirror_left_right();
-      break;
-    case Transformation::mirror_lr: mirror_left_right(); break;
-    case Transformation::mirror_tb: mirror_top_bottom(); break;
+    case Transformation::rotate90:
+      mirror_diagonal(); mirror_top_bottom( data ); break;
+    case Transformation::rotate180:
+      mirror_left_right( data ); mirror_top_bottom( data ); break;
+    case Transformation::rotate270:
+      mirror_diagonal(); mirror_left_right( data ); break;
+    case Transformation::mirror_lr: mirror_left_right( data ); break;
+    case Transformation::mirror_tb: mirror_top_bottom( data ); break;
     case Transformation::mirror_d1: mirror_diagonal(); break;
     case Transformation::mirror_d2:
-      mirror_diagonal(); mirror_left_right(); mirror_top_bottom(); break;
+      mirror_diagonal(); mirror_left_right( data ); mirror_top_bottom( data ); break;
     }
   }
