@@ -18,18 +18,18 @@
 
 // Only "P1" (pbm) and "P4" (pbm RAWBITS) file formats are recognized.
 
+class Track;
+
 class Bitmap
   {
-  int _width;
   int _height;
+  int _width;
   std::vector< std::vector< bool > > data;
   std::vector< Rectangle > _rectangle_vector;
 
   int  find_columns( int rindex ) throw();
   bool find_rows_and_columns() throw();
   void mirror_diagonal() throw();
-  void mirror_left_right() throw();
-  void mirror_top_bottom() throw();
   static void floodfill4( const Bitmap & source, Bitmap & dest,
                           int row, int col ) throw();
 public:
@@ -41,18 +41,23 @@ public:
 
   enum Type { vertical_histogram, horizontal_histogram, connected_ground };
 
-  // Creates a Bitmap from a pbm file
+	// Creates a Bitmap from a pbm file
   Bitmap( FILE * f, bool invert ) throw( Error );
-  // Creates a Bitmap from another Bitmap
+	// Creates a Bitmap from another Bitmap
   Bitmap( const Bitmap & source, const Rectangle & r ) throw();
-  // Creates a reduced bitmap
+	// Creates a reduced bitmap
   Bitmap( const Bitmap & source, int scale, int th ) throw();
-  // Creates a "special" type of Bitmap
-  Bitmap( const Bitmap &, Type ) throw( Error );
+	// Creates a "special" type of Bitmap
+  Bitmap( const Bitmap & source, Type t ) throw( Error );
+	// Creates a Bitmap from a matrix of int
+  Bitmap( const std::vector< std::vector< int > > & matrix ) throw();
 
-  int width()  const throw() { return _width;  }
   int height() const throw() { return _height; }
+  int width()  const throw() { return _width;  }
   bool get_bit( int row, int col ) const throw() { return data[row][col]; }
+  void set_bit( int row, int col, bool bit ) throw() { data[row][col] = bit; }
+  void draw_rectangle( const Rectangle & r ) throw();
+  void draw_track( const Track & t ) throw();
 
   const std::vector< Rectangle > & rectangle_vector() const throw()
     { return _rectangle_vector; }
