@@ -16,6 +16,7 @@
     Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#include <algorithm>
 #include <cstdio>
 #include <vector>
 #include "common.h"
@@ -24,6 +25,53 @@
 #include "blockmap.h"
 
 
+namespace {
+
+bool escape_left_right( const Block & b, int row, int col, bool to_left ) throw()
+  {
+  const Blockmap & bm = *b.blockmap();
+  const int id = b.id();
+  if( row < b.top() || row > b.bottom() || bm.id( row, col ) == id )
+    return false;
+  const int begin = std::min( row - b.top(), std::max( 1, b.height() / 20 ) );
+  const int end = std::max( b.bottom() - row, b.height() - begin - 1 );
+  if( end - begin + 1 <= 0 ) return false;
+  bool array[b.height()];
+
+  for( int i = row - b.top(); i >= begin; --i )
+    {
+    if( bm.id( b.top() + i, col ) != id ) array[i] = true;
+    else { while( i >= begin ) { array[i--] = false; } break; }
+    }
+  for( int i = row - b.top() + 1; i <= end; ++i )
+    {
+    if( bm.id( b.top() + i, col ) != id ) array[i] = true;
+    else { while( i <= end ) { array[i++] = false; } break; }
+    }
+
+  while( ( to_left && --col >= b.left() ) ||
+         ( !to_left && ++col <= b.right() ) )
+    {
+    bool alive = false;
+    for( int i = begin; i <= end; ++i )
+      if( array[i] )
+        { if( bm.id( b.top() + i, col ) == id ) array[i] = false;
+        else alive = true; }
+    if( !alive ) return false;
+
+    for( int i = begin + 1; i <= end; ++i )
+      if( array[i-1] && !array[i] && bm.id( b.top() + i, col ) != id )
+        array[i] = true;
+    for( int i = end - 1; i >= begin; --i )
+      if( !array[i] && array[i+1] && bm.id( b.top() + i, col ) != id )
+        array[i] = true;
+    }
+  return true;
+  }
+
+} // end namespace
+
+
 // Return the total filled area of this block (no recursive)
 int Block::area() const throw()
   {
@@ -82,39 +130,7 @@ bool Block::compare_id( int id, bool recursive ) const throw()
 
 
 bool Block::escape_left( int row, int col ) const throw()
-  {
-  const Blockmap & bm = *_blockmap;
-  if( bm.id( row, col ) == _id ) return false;
-  bool array[height()];
-
-  for( int r = row; r > top(); --r )
-    {
-    if( bm.id( r, col ) != _id ) array[r-top()] = true;
-    else { for( int i = r - top(); i >= 0; --i ) array[i] = false; break; }
-    }
-  for( int r = row + 1; r < bottom(); ++r )
-    {
-    if( bm.id( r, col ) != _id ) array[r-top()] = true;
-    else { for( int i = r - top(); i < height(); ++i ) array[i] = false; break; }
-    }
-
-  while( --col >= left() )
-    {
-    bool alive = false;
-    for( int i = 0; i < height(); ++i ) if( array[i] )
-      { if( bm.id( top() + i, col ) == _id ) array[i] = false;
-      else alive = true; }
-    if( !alive ) return false;
-
-    for( int i = 2; i < height() - 1; ++i )
-      if( array[i-1] && !array[i] &&
-          bm.id( top() + i, col ) != _id ) array[i] = true;
-    for( int i = height() - 3; i > 0; --i )
-      if( array[i+1] && !array[i] &&
-          bm.id( top() + i, col ) != _id ) array[i] = true;
-    }
-  return true;
-  }
+  { return escape_left_right( *this, row, col, true ); }
 
 
 bool Block::escape_top( int row, int col ) const throw()
@@ -137,39 +153,7 @@ bool Block::escape_top( int row, int col ) const throw()
 
 
 bool Block::escape_right( int row, int col ) const throw()
-  {
-  const Blockmap & bm = *_blockmap;
-  if( bm.id( row, col ) == _id ) return false;
-  bool array[height()];
-
-  for( int r = row; r > top(); --r )
-    {
-    if( bm.id( r, col ) != _id ) array[r-top()] = true;
-    else { for( int i = r - top(); i >= 0; --i ) array[i] = false; break; }
-    }
-  for( int r = row + 1; r < bottom(); ++r )
-    {
-    if( bm.id( r, col ) != _id ) array[r-top()] = true;
-    else { for( int i = r - top(); i < height(); ++i ) array[i] = false; break; }
-    }
-
-  while( ++col <= right() )
-    {
-    bool alive = false;
-    for( int i = 0; i < height(); ++i ) if( array[i] )
-      { if( bm.id( top() + i, col ) == _id ) array[i] = false;
-      else alive = true; }
-    if( !alive ) return false;
-
-    for( int i = 2; i < height() - 1; ++i )
-      if( array[i-1] && !array[i] &&
-          bm.id( top() + i, col ) != _id ) array[i] = true;
-    for( int i = height() - 3; i > 0; --i )
-      if( array[i+1] && !array[i] &&
-          bm.id( top() + i, col ) != _id ) array[i] = true;
-    }
-  return true;
-  }
+  { return escape_left_right( *this, row, col, false ); }
 
 
 bool Block::escape_bottom( int row, int col ) const throw()
@@ -421,6 +405,14 @@ bool Block::adjust_height() throw()
   }
 
 
+void Block::delete_block( int i ) throw()
+  {
+  if( i < 0 || i >= blocks() )
+    Ocrad::internal_error( "delete_block, index out of bounds" );
+  _block_vector.erase( _block_vector.begin() + i );
+  }
+
+
 void Block::shift_block( const Block & b ) throw()
   {
   int i = blocks() - 1;
@@ -448,14 +440,14 @@ void Block::hierarchize_blocks( std::vector< Block > & block_vector ) throw()
       if( b1.strictly_includes( b2 ) )
         {
         if( ( b1.id() > 0 && b2.id() < 0 ) || ( b1.id() < 0 && b2.id() > 0 ) )
-          { b1.shift_block( b2 ); shift = true; }
+          { shift = true; b1.shift_block( b2 ); }
         else
           {
           for( int i3 = 0; i3 < b1.blocks(); ++i3 )
             {
             Block & b3 = b1._block_vector[i3];
             if( b3.strictly_includes( b2 ) )
-              { b3.shift_block( b2 ); shift = true; }
+              { shift = true; b3.shift_block( b2 ); }
             }
           }
         }
