@@ -20,30 +20,37 @@ class Bitmap;
 
 class Blockmap
   {
+public:
+  struct Zone
+    {
+    Rectangle rectangle;
+    std::vector< std::vector< Block > > block_matrix;
+    Zone( const Rectangle & r ) : rectangle( r ) {}
+    };
+
+private:
   int _height;
   int _width;
   std::vector< std::vector< int > > data;
-  std::vector< std::vector< Block > > _block_matrix;
+  std::vector< Zone > _zone_vector;
 
   int generate_black_id();
   int generate_white_id();
 
-  void add_point_to_block( int row, int col, int id ) throw();
-  void create_block( const Rectangle & r, int id ) throw();
-  void delete_block( int id ) throw();
-  void join_blocks( int id1, int id2 ) throw();
-
-  void ignore_wide_blocks() throw();
-
 public:
-  Blockmap( const Bitmap & source, int rindex, int debug_level ) throw();
+  Blockmap( const Bitmap & source, int debug_level ) throw();
 
   int height() const throw() { return _height; }
   int width()  const throw() { return _width;  }
   int id( int row, int col ) const throw() { return data[row][col]; }
-  const std::vector< std::vector< Block > > & block_matrix() const throw()
-    { return _block_matrix; }
-  int blocks() const throw();
+  const std::vector< std::vector< int > > & data_matrix() const throw() { return data; }
+
+  int zones() const throw() { return _zone_vector.size(); }
+  const std::vector< Zone > & zone_vector() const throw() { return _zone_vector; }
+  int blocks_in_map() const throw();
+  int blocks_in_zone( int zindex ) const throw();
+  int blocks_in_cut( int zindex, int cut ) const throw()
+    { return _zone_vector[zindex].block_matrix[cut].size(); }
 
   void print( FILE * outfile, int debug_level ) const throw();
   };
