@@ -33,7 +33,7 @@ class Character : public Rectangle
   void recognize11( const Charset & charset, int charbox_vcenter ) throw();
     void recognize110( const Charset & charset, int charbox_vcenter ) throw();
     void recognize111( const Charset & charset, int charbox_vcenter ) throw();
-    void recognize112() throw();
+    void recognize112( int charbox_vcenter ) throw();
   void recognize12( const Charset & charset, int charbox_vcenter ) throw();
   void recognize13( const Charset & charset ) throw();
 
@@ -46,7 +46,7 @@ public:
     : Rectangle( r ), _blockmap( &b ) {}
 
   int   area() const throw();
-  const Block & block( int i ) const throw();
+  Block & block( int i ) throw();
   int   blocks() const throw() { return _block_vector.size(); }
   const Blockmap * blockmap() const throw() { return _blockmap; }
 
