@@ -16,6 +16,7 @@
     Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#include <algorithm>
 #include <cstdio>
 #include <vector>
 #include "common.h"
@@ -45,7 +46,7 @@ void Character::recognize11( const Charset & charset, int charbox_vcenter ) thro
   const Block & b = block( 0 );
   if( b.blocks() == 0 ) recognize110( charset, charbox_vcenter );
   else if( b.blocks() == 1 ) recognize111( charset, charbox_vcenter );
-  else if( b.blocks() == 2 ) recognize112();
+  else if( b.blocks() == 2 ) recognize112( charbox_vcenter );
   }
 
 
@@ -124,15 +125,15 @@ void Character::recognize110( const Charset & charset, int charbox_vcenter ) thr
           }
         }
       }
-    else if( code == 'h' )	// Looks for merged 'rf'
+    else if( code == 'h' )	// Looks for merged 'rf' or 'fi'
       {
       if( b.seek_right( b.top() + 1, b.left() ) > b.hcenter() )
         {
         if( f.rp[f.rp.pos(70)] >= 2 &&
-            b.seek_top( b.vpos(70), b.right() ) > b.top() )
+            b.seek_top( b.vpos( 70 ), b.right() ) > b.top() )
           {
           int col = 0, hmin = f.hp.range() + 1;
-          for( int i = b.hpos(40); i <= b.hpos(60); ++i )
+          for( int i = b.hpos( 40 ); i <= b.hpos( 60 ); ++i )
             if( f.hp[i-b.left()] < hmin )
               { hmin = f.hp[i-b.left()]; col = i; }
           if( col > b.left() && col < b.right() )
@@ -141,6 +142,11 @@ void Character::recognize110( const Charset & charset, int charbox_vcenter ) thr
           }
         return;
         }
+      if( f.rp.isctip( 33 ) )
+        {
+        only_guess( 0, b.left() ); add_guess( 'f', b.hcenter() );
+        add_guess( 'i', b.right() ); return;
+        }
       }
     else if( code == 'k' )	// Looks for merged 'rt'
       {
@@ -152,12 +158,27 @@ void Character::recognize110( const Charset & charset, int charbox_vcenter ) thr
       }
     if( code ) { add_guess( code, 0 ); return; }
     }
-  if( f.bp.minima() == 3 ) { add_guess( 'm', 0 ); return; }
-  if( f.bp.minima() == 4 )
+  if( f.bp.minima() == 3 )
     {
-    int col = b.seek_right( b.bottom() - 1, b.left() );
-    col = b.seek_right( b.bottom() - 1, col + 1, false );
-    col = b.seek_right( b.bottom() - 1, col + 1 );
+    if( _blockmap->id( b.vcenter(), b.hcenter() ) == 0 &&
+        b.seek_left( b.vcenter(), b.hcenter() ) <= b.hpos( 25 ) )
+      {					// Found merged 'rn'
+      int row = b.vpos( 95 );
+      int col = b.seek_right( row, b.left() );
+      col = b.seek_right( row, col + 1, false );
+      col = b.seek_right( row, col + 1 );
+      if( col > b.left() && col < b.right() )
+        { only_guess( 0, b.left() ); add_guess( 'r', col );
+        add_guess( 'n', b.right() ); return; }
+      }
+    add_guess( 'm', 0 ); return;
+    }
+  if( f.bp.minima() == 4 && f.tp.minima( b.height() / 3 ) == 1 )
+    {					// Found merged 'rm'
+    int row = b.vpos( 95 );
+    int col = b.seek_right( row, b.left() );
+    col = b.seek_right( row, col + 1, false );
+    col = b.seek_right( row, col + 1 );
     if( col > b.left() && col < b.right() )
       { only_guess( 0, b.left() ); add_guess( 'r', col );
       add_guess( 'm', b.right() ); return; }
@@ -183,16 +204,18 @@ void Character::recognize111( const Charset & charset, int charbox_vcenter ) thr
   {
   const Block & b = block( 0 );
   const Block & h = b.block( 0 );			// hole
+  if( !h.is_hcentered_in( b ) ) return;
   Features f( b );
   int top_delta = h.top() - b.top(), bottom_delta = b.bottom() - h.bottom();
 
-  if( std::abs( top_delta - bottom_delta ) <= 2 ||
-      Ocrad::similar( top_delta, bottom_delta, 40 ) ) // hole is vertically centered
-    {
+  if( std::abs( top_delta - bottom_delta ) <= std::max( 2, h.height() / 4 ) ||
+      Ocrad::similar( top_delta, bottom_delta, 40, 2 ) )
+    {					// hole is vertically centered
     int code = f.test_4ADQao( charset, charbox_vcenter );
     if( code )
       {
-      if( code == 'Q' ) add_guess( 'a', 1 );
+      if( code == 'Q' && Ocrad::similar( top_delta, bottom_delta, 40, 2 ) )
+        add_guess( 'a', 1 );
       add_guess( code, 0 );
       }
     return;
@@ -227,17 +250,15 @@ void Character::recognize111( const Charset & charset, int charbox_vcenter ) thr
 
 // Recognizes 1 block characters with 2 holes.
 // 8BQg$&
-void Character::recognize112() throw()
+void Character::recognize112( int charbox_vcenter ) throw()
   {
   const Block & b = block( 0 );
   const Block & h1 = b.block( 0 );		// upper hole
   const Block & h2 = b.block( 1 );		// lower hole
+  if( !h1.is_hcentered_in( b ) ) return;
+  if( !h2.is_hcentered_in( b ) ) return;
   int a1 = h1.area();
   int a2 = h2.area();
-//  Features f( b );
-
-  if( h1.includes_vcenter( h2 ) && h2.includes_vcenter( h1 ) )
-    { add_guess( 'm', 0 ); return; }
 
   Profile lp( b, Profile::left );
   Profile tp( b, Profile::top );
@@ -274,7 +295,12 @@ void Character::recognize112() throw()
       else if( rp.ispit() ) add_guess( 'a', 1 );
       add_guess( '8', 0 ); return;
       }
-    if( lp.minima() == 2 && rp.minima() == 1 ) { add_guess( 'a', 0 ); return; }
+    if( lp.minima() == 2 && rp.minima() == 1 )
+      {
+      if( charbox_vcenter < h1.bottom() ) add_guess( 'g', 0 );
+      else add_guess( 'a', 0 );
+      return;
+      }
     add_guess( 'B', 1 ); add_guess( 'a', 0 ); return;
     }
   if( a1 > a2 && h1.h_overlaps( h2 ) )
@@ -290,6 +316,7 @@ void Character::recognize112() throw()
       if( 2 * h1.height() > h2.height() && 2 * h1.width() > h2.width() &&
           3 * h2.width() >= b.width() && !lp.isctip() )
         add_guess( 'B', 0 );
+      else if( h2.right() < b.hcenter() ) add_guess( '&', 0 );
       else add_guess( 'a', 0 );
       return;
       }
