@@ -231,7 +231,7 @@ int Features::test_235Esz( const Charset & charset ) const throw()
           else if( black_section == 2 ) { lrow2 = row; lcol2 = col; } }
       prev_black = black;
       }
-    if( black_section == 3 && lrow1 <= b.vpos(30) && urow3 >= b.vpos(70) )
+    if( black_section == 3 && lrow1 <= b.vpos( 30 ) && urow3 >= b.vpos( 70 ) )
       break;
     black_section = 0;
     }
@@ -282,8 +282,8 @@ int Features::test_235Esz( const Charset & charset ) const throw()
           return 'F';
           }
         if( 8 * lp[((lrow2+urow3)/2)-b.top()] >= b.width() &&
-            b.escape_top( urow2, b.left() ) &&
-            !b.escape_top( lrow2, b.left() ) ) return 'f';
+            b.escape_top( ( lrow1 + urow2 ) / 2, b.left() ) &&
+            !b.escape_top( ( lrow2 + urow3 ) / 2, b.left() ) ) return 'f';
         else if( lrow1 < urow2 && lrow2 < urow3 ) return 'E';
         }
       }
@@ -291,9 +291,7 @@ int Features::test_235Esz( const Charset & charset ) const throw()
       {
       if( ( tp.isconvex() || ( tp.ispit() && !bp.ispit() ) ) &&
           lp[lp.pos(50)] >= b.width() / 2 ) return '2';
-      if( b.height() > 2 * wp.max() && bp.minima( b.height() / 5 ) == 1 )
-        { if( b.height() >= 3 * wp.max() ) return 'l'; else return 'I'; }
-      return 'z';
+      if( b.height() <= 2 * wp.max() ) return 'z';
       }
     }
   return 0;
@@ -322,7 +320,7 @@ int Features::test_s_cedilla() const throw()
         }
       prev_black = black;
       }
-    if( black_section == 4 && urow2 < b.vpos(50) && urow4 >= b.vpos(70) )
+    if( black_section == 4 && urow2 < b.vpos( 50 ) && urow4 >= b.vpos( 70 ) )
       break;
     black_section = 0;
     }
@@ -339,14 +337,14 @@ int Features::test_s_cedilla() const throw()
 int Features::test_49ARegpq() const throw()
   {
   const Block & b = *_block;
+  const Block & h = b.block( 0 );			// hole
 
   if( bp.minima( b.height() / 10 ) == 2 && bp.isctip() && tp.minima() == 1 )
     { if( tp.isvpit() ) return 'A'; else return 'R'; }
 
-  const Block & h = b.block( 0 );			// hole
   int col = h.hcenter();
   int row = b.seek_bottom( h.bottom(), col, false ) + 1;
-  if( row >= b.vpos(90) )
+  if( row >= b.vpos( 90 ) )
     { col = h.left(); row = b.seek_bottom( h.bottom(), col, false ) + 1; }
   if( row >= b.bottom() ) return 0;
 
@@ -358,13 +356,13 @@ int Features::test_49ARegpq() const throw()
     if( bp.ispit() )
       {
       int row2 = b.seek_bottom( row, h.right() );
-      if( row2 < b.vpos(75) ) return 'g';
+      if( row2 < b.vpos( 75 ) ) return 'g';
       if( row2 < b.bottom() ) return 'e';
       }
     if( bp.minima() == 1 ) return 'p';
     }
 
-  if( b.escape_left( row, col ) )
+  else if( b.escape_left( row, col ) )
     {
     Profile hlp( h, Profile::left );
     Profile htp( h, Profile::top );
@@ -376,10 +374,10 @@ int Features::test_49ARegpq() const throw()
     int hdiff;
     if( b.bottom_hook( &hdiff ) && hdiff > 0 )
       { if( rp.isctip() ) return 's'; else return 'g'; }
-    if( row > b.vpos(80) ) return 'Q';
+    if( row > b.vpos( 80 ) && tp.ispit() ) return 'Q';
     if( b.seek_bottom( row, col ) < b.bottom() &&
         rp.increasing( row - b.top() ) ) return 'g';
-    return 'q';
+    if( bp.minima() == 1 ) return 'q';
     }
   return 0;
   }
@@ -417,12 +415,15 @@ int Features::test_4ADQao( const Charset & charset, int charbox_vcenter ) const
       if( c == 4 && Ocrad::similar( left_delta, right_delta, 40 ) &&
           lp.ispit() ) return '@';
       }
-    if( tp.minima() == 1 && bp.istip() && !lp.isconvex() ) return 'A';
+    if( tp.minima() == 1 && bp.istip() && !lp.isconvex() && !rp.isctip( 66 ) )
+      return 'A';
     }
-  if( Ocrad::similar( left_delta, right_delta, 40 ) )
+  if( Ocrad::similar( left_delta, right_delta, 50 ) )
     {
     if( !bp.isconvex() && rp.ispit() && bp[1] < 2 ) return 'D';
-    if( bp.minima() > 1 || rp.minima() > 1 ) return 'Q';
+    if( bp.minima() > 1 || rp.minima() > 1 )
+      { if( 4 * h.size() >= b.size() || tp.ispit() || lp.ispit() ) return 'Q';
+      else return 0; }
     if( b.vpos( 80 ) < charbox_vcenter ) return UCS::DEG;
     return 'o';
     }
@@ -460,15 +461,14 @@ int Features::test_6abd( const Charset & charset ) const throw()
     {
     int c = lp[urow-b.top()];
     bool oacute2 = ( c > lp[h.top()-b.top()] + 1 &&
-                     tp[std::min(c-1,b.width()/4)] + b.top() >= urow );
-    if( charset.enabled( Charset::iso_8859_15 ) ||
-        charset.enabled( Charset::iso_8859_9 ) )
-      if( oacute1 && oacute2 )
-        {
-        bool oacute3 = ( b.seek_left( b.top() + 1, b.right() + 1 ) >= rcol ||
-                         b.seek_right( h.top(), b.left() - 1 ) <= b.left() + 1 );
-        if( oacute3 ) return UCS::SOACUTE;
-        }
+                   urow <= b.top() + tp[std::min( c - 1, b.width() / 4 )] );
+    if( ( oacute1 && oacute2 ) && ( charset.enabled( Charset::iso_8859_15 ) ||
+                                    charset.enabled( Charset::iso_8859_9 ) ) )
+      {
+      bool oacute3 = ( b.right() - rp[rp.pos(5)] >= h.right() ||
+                       b.left() + lp[h.top()-b.top()] <= b.hpos( 5 ) );
+      if( oacute3 ) return UCS::SOACUTE;
+      }
     if( !oacute2 && lp.ispit() && bp.ispit() )
       {
       int row2 = b.seek_top( h.top(), h.right() + 1, false ) - 1;
@@ -488,7 +488,10 @@ int Features::test_6abd( const Charset & charset ) const throw()
       {
       if( charset.enabled( Charset::iso_8859_15 ) ||
           charset.enabled( Charset::iso_8859_9 ) )
-        if( b.vcenter() < row ) return UCS::SAACUTE;
+        {
+        int row3 = b.seek_top( row, col );
+        if( row > b.vcenter() && row3 > b.vpos( 20 ) ) return UCS::SAACUTE;
+        }
       return 'a';
       }
     if( charset.enabled( Charset::iso_8859_15 ) ||
@@ -512,8 +515,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
     return 0;
 
   int row, col;
-  if( b.seek_right( b.vcenter(), b.left() - 1 ) >= b.hcenter() )
-    col = b.hpos( 25 );
+  if( 2 * lp[lp.pos(50)] >= b.width() ) col = b.hpos( 25 );
   else col = b.hpos( 75 );
   row = b.seek_top( b.vcenter(), col );
   if( row <= b.top() || ( row < b.vpos( 25 ) && b.escape_top( row, col ) ) )
@@ -521,11 +523,13 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
     int hdiff;
     if( b.bottom_hook( &hdiff ) )
       {
-      if( hdiff > b.height() / 2 && rp.increasing( rp.pos(75) ) ) return 'J';
-      if( -hdiff > b.height() / 2 ) return 'L';
+      if( hdiff > b.height() / 2 && rp.increasing( rp.pos( 75 ) ) ) return 'J';
+      if( -hdiff > b.height() / 2 )
+        { if( 5 * lp[lp.pos(80)] < 2 * b.width() ) return 'L'; else return 'v'; }
       }
     }
 
+  int topmax = b.top() + 1 + ( b.width() / 30 );
   if( vbars() == 1 && vbar(0).width() >= 2 &&
       3 * vbar(0).width() <= b.width() + 2 )
     {
@@ -534,12 +538,12 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
       {
       if( hbars() == 1 && 4 * hbar(0).height() <= b.height() )
         {
-        if( hbar(0).top() <= b.top() + 1 ) return 'T';
+        if( hbar(0).top() <= topmax ) return 'T';
         if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
             Ocrad::similar( b.height(), b.width(), 50 ) ) return '+';
         }
       if( hbars() == 2 && 4 * hbar(0).height() <= b.height() &&
-          hbar(0).top() <= b.top() + 1 &&
+          hbar(0).top() <= topmax &&
           hbar(1).bottom() >= b.bottom() - 1 &&
           3 * hbar(0).width() > 4 * hbar(1).width() )
         return 'T';
@@ -551,7 +555,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
     {
     if( vbar(0).right() <= b.hcenter() )
       {
-      if( hbars() == 3 && hbar(0).top() <= b.top() + 1 &&
+      if( hbars() == 3 && hbar(0).top() <= topmax &&
           hbar(0).width() + 1 >= hbar(1).width() &&
           2 * hbar(1).width() >= 3 * vbar(0).width() &&
           vbar(0).h_overlaps( hbar(1) ) )
@@ -566,7 +570,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
         }
       if( hbars() == 2 )
         {
-        if( hbar(0).top() <= b.top() + 1 &&
+        if( hbar(0).top() <= topmax &&
             hbar(0).width() + 1 >= hbar(1).width() &&
             2 * hbar(1).width() >= 3 * vbar(0).width() &&
             hbar(1).includes_vcenter( b ) )
@@ -584,7 +588,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
       }
 
     if( vbar(0).left() > b.hcenter() &&
-        hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
+        hbars() == 1 && hbar(0).top() <= topmax &&
         hbar(0).width() + 1 >= b.width() )
       {
       if( charset.enabled( Charset::iso_8859_15 ) ||
@@ -601,7 +605,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
         Ocrad::similar( vbar(0).left() - b.left(),
                         b.right() - vbar(0).right(), 30, 2 ) )
       {
-      if( rp.istip() && lp.istip() )
+      if( rp.istip() && lp.istip() && b.height() <= 3 * wp.max() )
         {
         if( b.height() <= 3 * b.width() &&
             lp[lp.pos(40)] > lp[lp.pos(60)] + 1 &&
@@ -611,10 +615,11 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
       if( rp.isflats() && ( lp.istip() || lp.isflats() ) ) return 'l';
       if( b.height() > 3 * wp.max() )
         {
-        if( rp.istip() && lp.ispit() && Ocrad::similar( lp.iminimum(), rp.pos(50), 10 ) )
+        if( rp.istip() && lp.ispit() && Ocrad::similar( lp.iminimum(), rp.pos( 50 ), 10 ) )
           { if( lp.isvpit() ) return '{'; else return '('; }
-        if( lp.istip() && rp.ispit() && Ocrad::similar( rp.iminimum(), rp.pos(50), 10 ) )
+        if( lp.istip() && rp.ispit() && Ocrad::similar( rp.iminimum(), rp.pos( 50 ), 10 ) )
           { if( rp.isvpit() ) return '}'; else return ')'; }
+        if( rp.isflats() && 2 * vbar(0).size() >= b.area() ) return 'l';
         }
       if( 2 * b.height() > 3 * b.width() )
         if( rp.isflats() || rp.minima() == 1 )
@@ -644,14 +649,15 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
       {
       if( lp.istip() && b.height() > 2 * b.width() )
         {
-        if( 2 * vbar(0).width() <= wp.max() ) return ']';
+        if( 2 * vbar(0).width() <= wp.max() &&
+            lp[lp.pos(50)] >= b.width() / 2 ) return ']';
         if( b.height() >= 3 * b.width() ) return 'l';
         }
       if( 2 * b.height() >= 3 * b.width() &&
           vbar(0).height() >= 3 * vbar(0).width() &&
           lp.isvpit() && lp.minima() == 1 )
         { int i = lp.iminimum();
-          if( i > lp.pos(20) && i < lp.pos(40) ) return '1'; }
+          if( i > lp.pos( 20 ) && i < lp.pos( 40 ) ) return '1'; }
       }
     }
   if( hbars() == 1 && std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
@@ -671,7 +677,9 @@ int Features::test_c() const throw()
     if( b.height() > 2 * b.width() )
       { if( 2 * b.width() >= 3 * wp.max() ) return '('; else return 0; }
     if( b.seek_bottom( b.vcenter(), b.hcenter() ) < b.bottom() &&
-        b.escape_right( b.vcenter(), b.hcenter() ) ) return 'c';
+        b.seek_top( b.vcenter(), b.hcenter() ) > b.top() &&
+        b.escape_right( b.vcenter(), b.hcenter() ) )
+      return 'c';
     }
   if( lp.ispit() && bp.ispit() && tp.ispit() &&
       b.escape_right( b.vcenter(), b.hcenter() ) )
@@ -725,11 +733,11 @@ int Features::test_frst() const throw()
           {
           if( tp.minima( b.height() / 8 ) == 2 &&
               bp.minima( b.height() / 8 ) == 2 ) return 'x';
-          int row2 = b.vpos(75);
+          int row2 = b.vpos( 75 );
           int col2 = b.seek_right( row2, b.hcenter(), false ) + 1;
           if( b.seek_right( row2, col2 ) >= b.right() )
             {
-            if( lp.isconvex() && ( col > b.hpos(60) || row < b.bottom() ) )
+            if( lp.isconvex() && ( col > b.hpos( 60 ) || row < b.bottom() ) )
               return 0;
             if( hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
                 4 * hbar(0).height() <= b.height() &&
@@ -751,14 +759,14 @@ int Features::test_frst() const throw()
       {
       int col = b.right() - rp[b.vcenter() - b.top()] + 1;
       if( !b.escape_bottom( b.vcenter(), col ) )
-        { if( tp.ispit() && lp.iminimum() < lp.pos(40) ) return 't'; }
+        { if( tp.ispit() && lp.iminimum() < lp.pos( 40 ) ) return 't'; }
       else if( 2 * wp.max() > b.width() )
         {
-        if( rp.iminimum() < rp.pos(20) )
+        if( rp.iminimum() < rp.pos( 20 ) )
           {
-          if( rp.increasing( rp.pos(20) ) || bp.increasing() ||
+          if( rp.increasing( rp.pos( 20 ) ) || bp.increasing() ||
               tp.minima( 1 ) == 2 ||
-              ( rp.minima() == 1 && tp.iminimum() > tp.pos(50) ) ) return 'r';
+              ( rp.minima() == 1 && tp.iminimum() > tp.pos( 50 ) ) ) return 'r';
           else return 'f';
           }
         else return 't';
@@ -841,7 +849,7 @@ int Features::test_HKMNUuvwYy() const throw()
               5 * wp[wp.pos(98)] >= 4 * wp[wp.pos(70)] ) ||
             ( lg > 1 && lg < rg && !rp.increasing() ) ) return 'Y';
       if( 2 * lg < rg && lp.isctip() ) return 'y';
-      if( b.escape_top( b.vpos(60), b.hcenter() ) ) return 'u';
+      if( b.escape_top( b.vpos( 60 ), b.hcenter() ) ) return 'u';
       if( lg < rg + 1 && !lp.increasing() && rp.increasing() &&
           ( tp.minima( b.height()/2 ) == 1 || lp.imaximum() > b.height()/2 ) )
         return 'y';
@@ -853,7 +861,8 @@ int Features::test_HKMNUuvwYy() const throw()
       }
     if( bp.minima() == 2 && bp.isctip() )
       {
-      if( bp.minima( bp[bp.pos(50)] + 1 ) == 3 ) return 'M';
+      int th = std::max( b.height() / 4, bp[bp.pos(50)] + 1 );
+      if( bp.minima( th ) == 3 ) return 'M';
       int lg = lp[lp.pos(50)];
       int rg = rp[rp.pos(50)];
       if( Ocrad::similar( lg, rg, 80, 2 ) &&
@@ -867,7 +876,8 @@ int Features::test_HKMNUuvwYy() const throw()
           return 'H';
         if( vscan()[rp.pos(60)] == 4 || vscan()[rp.pos(70)] == 4 ) return 'w';
         if( 2 * b.width() > 5 * b.height() ) return '~';
-        if( vbars() <= 2 || ( vbars() == 3 && b.height() >= b.width() ) )
+        if( ( vbars() <= 2 || ( vbars() == 3 && b.height() >= b.width() ) ) &&
+            std::abs( rp[rp.pos(25)] - rp[rp.pos(75)] ) <= b.width() / 4 )
           return 'N';
         return 0;
         }
@@ -897,7 +907,7 @@ int Features::test_hknwx() const throw()
   if( row < 2 ) return 0;
   row = b.bottom() - row + 1; col += b.left();
   if( row <= b.top() ) return 0;
-  //FIXME
+  // FIXME
   { int c = col; col = b.seek_right( row, col ); if( col > c ) --col;
   row = b.seek_top( row, col ); }
 
@@ -905,9 +915,11 @@ int Features::test_hknwx() const throw()
   int urow = b.seek_top( row - 1, col, false );
   if( urow > b.vpos( 20 ) )
     {
-    if( tp.minima() >= 2 && tp.minima( b.height() / 5 ) >= 2 ) return 'w';
+    if( tp.minima() >= 2 && tp.minima( b.height() / 5 ) >= 2 &&
+        rp[rp.pos(25)] <= b.width() / 4 ) return 'w';
     if( tp.minima( b.height() / 5 ) == 1 )
       { if( rp.isctip( 66 ) ) return 'k'; else return 'h'; }
+    return 0;
     }
   if( Ocrad::similar( b.height(), b.width(), 40 ) && row > b.vcenter() &&
       urow < b.vcenter() && tp.minima( b.height() / 5 ) == 2 &&
@@ -954,7 +966,7 @@ int Features::test_easy( int charbox_vcenter ) const throw()
   if( b.bottom() <= charbox_vcenter &&
       b.height() > b.width() && bp.minima() == 1 )
     {
-    if( tp.iminimum() < tp.pos(50) && bp.iminimum() > bp.pos(50) )
+    if( tp.iminimum() < tp.pos( 50 ) && bp.iminimum() > bp.pos( 50 ) )
       return '`'; else return '\'';
     }
   if( 2 * b.height() > 3 * wp.max() && b.top() >= charbox_vcenter &&
@@ -1087,7 +1099,8 @@ int Features::test_misc() const throw()
           hbar(1).bottom() >= b.bottom() - 1 )
         {
         if( hbar(0).right() <= hbar(1).hcenter() ) return 0;
-        if( 3 * hbar(0).width() <= 2 * hbar(1).width() ) return 'l';
+        if( 3 * hbar(0).width() <= 2 * hbar(1).width() ||
+            b.height() > 3 * wp.max() ) return 'l';
         return 'I';
         }
       }
@@ -1096,7 +1109,7 @@ int Features::test_misc() const throw()
         3 * hbar(0).width() > 4 * hbar(1).width() ) )
       if( 3 * hbar(0).height() < b.height() && hbar(0).top() <= b.top() + 1 )
         {
-        int i = lp.pos(40);
+        int i = lp.pos( 40 );
         if( 3 * wp[i] < b.width() && 5 * lp[i] > b.width() &&
             5 * rp[i] > b.width() ) return 'T';
         }
