@@ -37,22 +37,21 @@
 
 namespace {
 
-// Date of this version: 2004-10-23
+// Date of this version: 2004-12-09
 
-const char * const program_version = "0.9";
+const char * const program_version = "0.10";
 const char * const program_year    = "2004";
 
 struct Input_control
   {
   Transformation transformation;
   int rindex, layout_level, scale, threshold;
-  char filetype, specialtype;
+  char specialtype;
   bool copy, invert;
 
   Input_control() throw()
     : rindex( -1 ), layout_level( 0 ), scale( 0 ), threshold( -1 ),
-      filetype( 0 ), specialtype( 0 ),
-      copy( false ), invert( false )
+      specialtype( 0 ), copy( false ), invert( false )
     {}
   bool set_format( const char * name ) throw();
   };
@@ -141,7 +140,7 @@ int process_file( FILE *infile, const char * infile_name,
       if( control.outfile )
         {
         Bitmap reduced( page_image, input_control.scale, input_control.threshold );
-        reduced.save( control.outfile, input_control.filetype );
+        reduced.save( control.outfile, control.filetype );
         }
       return 0;
       }
@@ -155,7 +154,7 @@ int process_file( FILE *infile, const char * infile_name,
         else if( input_control.specialtype == 'g' ) t = Bitmap::connected_ground;
         else { show_error( 0, "bad special type" ); return 1; }
         Bitmap sb( page_image, t );
-        sb.save( control.outfile, input_control.filetype );
+        sb.save( control.outfile, control.filetype );
         }
       return 0;
       }
@@ -164,11 +163,11 @@ int process_file( FILE *infile, const char * infile_name,
       if( control.outfile )
         {
         if( input_control.layout_level == 0 && page_image.rectangles() == 1 )
-          { page_image.save( control.outfile, input_control.filetype ); return 0; }
+          { page_image.save( control.outfile, control.filetype ); return 0; }
         for( int c = 0; c < page_image.rectangles(); ++c )
           {
           Bitmap bitmap( page_image, page_image.rectangle_vector()[c] );
-          bitmap.save( control.outfile, input_control.filetype );
+          bitmap.save( control.outfile, control.filetype );
           }
         }
       return 0;
@@ -176,16 +175,7 @@ int process_file( FILE *infile, const char * infile_name,
 
     Textpage textpage( page_image, my_basename( infile_name ), control );
 
-    if( control.debug_level >= 86 && control.debug_level <= 89 )
-      {
-      if( control.outfile )
-        {
-        bool graph = ( control.debug_level >= 88 );
-        bool recursive = ( control.debug_level & 1 );
-        textpage.dprint( control, graph, recursive );
-        }
-      }
-    else if( control.debug_level == 0 )
+    if( control.debug_level == 0 )
       {
       if( control.outfile ) textpage.print( control );
       if( control.exportfile ) textpage.xprint( control );
@@ -241,7 +231,7 @@ int main( int argc, char *argv[] ) throw()
       case 0: break;
       case '?': show_error( argv[0], 0 ); return 1;	// bad option
       case '1':
-      case '4': input_control.filetype = c; break;
+      case '4': control.filetype = c; break;
       case 'C': input_control.copy = true; break;
       case 'D': control.debug_level = std::strtol( optarg, 0, 0 ); break;
       case 'F': if( !control.set_format( optarg ) )
