@@ -16,6 +16,7 @@
     Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#include <algorithm>
 #include <cstdio>
 #include <vector>
 #include "common.h"
@@ -26,12 +27,12 @@
 
 
 Profile::Profile( const Block & b, Type t ) throw()
-  : _block( &b ), _blockmap( b.blockmap() ), type(t), valid(false),
-    valid_max(false), valid_min(false), valid_mean(false),
-    valid_range(false), valid_isconcave(false),
-    valid_isconvex(false), valid_isflat(false), valid_isflats(false),
-    valid_ispit(false), valid_isupit(false), valid_isvpit(false),
-    valid_istip(false) {}
+  : _block( &b ), _blockmap( b.blockmap() ), type( t ), valid( false ),
+    valid_max( false ), valid_min( false ), valid_mean( false ),
+    valid_isconcave( false ), valid_isconvex( false ),
+    valid_isflat( false ), valid_isflats( false ),
+    valid_ispit( false ), valid_isupit( false ), valid_isvpit( false ),
+    valid_istip( false ) {}
 
 
 void Profile::initialize() throw()
@@ -109,12 +110,6 @@ int Profile::mean() throw()
   }
 
 
-int Profile::limit() throw()
-  {
-  if( !valid ) initialize(); return _limit;
-  }
-
-
 int Profile::max() throw()
   {
   if( !valid_max )
@@ -147,22 +142,6 @@ int Profile::operator[]( int i ) throw()
   return data[i];
   }
 
-int Profile::pos( int p ) throw()
-  { return ( ( samples() - 1 ) * p ) / 100; }
-
-
-int Profile::range() throw()
-  {
-  if( !valid_range )
-    {
-    valid_range = true; if( !valid ) initialize();
-    _range = 0;
-    for( int i = 0; i < samples(); ++i ) if( data[i] > _range )
-      _range = data[i];
-    }
-  return _range;
-  }
-
 
 bool Profile::increasing( int i ) throw()
   {
@@ -255,10 +234,12 @@ bool Profile::isflat() throw()
   if( !valid_isflat )
     {
     valid_isflat = true; _isflat = false; if( !valid ) initialize();
-    int th = ( mean() < 1 ) ? 1 : mean();
-    for( int i = 2; i < samples() - 2; ++i )
-      if( std::abs( data[i] - th ) > 1 ) return _isflat;
-    _isflat = true;
+    if( samples() < 15 ) return _isflat;
+    int mn = data[samples()/2], mx = mn;
+
+    for( int i = 1; i < samples() - 1; ++i )
+      { int d = data[i]; if( d < mn ) mn = d; else if( d > mx ) mx = d; }
+    _isflat = ( mx - mn <= 1 + ( samples() / 30 ) );
     }
   return _isflat;
   }
@@ -269,21 +250,19 @@ bool Profile::isflats() throw()
   if( !valid_isflats )
     {
     valid_isflats = true; _isflats = false; if( !valid ) initialize();
-    int begin = pos( 10 ); if( begin < 3 ) begin = 3;
-    int end = pos( 90 ); if( end > samples() - 4 ) end = samples() - 4;
-    if( 2 * ( end - begin ) <= samples() ) return _isflats;
+    if( samples() < 15 ) return _isflats;
+    const int s1 = std::max( pos( 15 ), 3 );
+    const int s2 = std::min( pos( 85 ), samples() - 4 );
+    int mn = -1, mx = 0;
+    for( int i = s1 + 2; i < s2; ++i )
+      if( data[i-1] == data[i] ) { mn = mx = data[i]; break; }
+    if( mn < 0 ) return _isflats;
 
-    int mean1 = 0;
-    for( int i = begin; i < end; ++i ) mean1 += data[i];
-    mean1 /= ( end - begin );
-
-    for( int i = 1; i < begin; ++i )
-      if( data[i] - mean1 > 1 ) return _isflats;
-    for( int i = begin; i < end; ++i )
-      if( std::abs( data[i] - mean1 ) > 1 ) return _isflats;
-    for( int i = end; i < samples() - 1; ++i )
-      if( data[i] - mean1 > 1 ) return _isflats;
-    _isflats = true;
+    for( int i = 1; i <= s1; ++i ) if( data[i] > mx ) mx = data[i];
+    for( int i = s1 + 1; i < s2; ++i )
+      { int d = data[i]; if( d < mn ) mn = d; else if( d > mx ) mx = d; }
+    for( int i = s2; i < samples() - 1; ++i ) if( data[i] > mx ) mx = data[i];
+    _isflats = ( mx - mn <= 1 + ( samples() / 30 ) );
     }
   return _isflats;
   }
@@ -415,13 +394,18 @@ bool Profile::istip() throw()
     if( samples() < 5 ) return _istip;
 
     int th = ( mean() < 2 && range() > 2 ) ? 2 : mean(); if( th < 2 ) ++th;
-    int thu = std::min( data[0], data[1] );
-    if( thu >= th ) return _istip;
-    int thd = std::min( data[samples()-1], data[samples()-2] );
-    if( thd >= th ) return _istip;
-    th = std::max( thu, thd );
+    int lth = data[0], rth = data[samples()-1];
+    int begin = 0, end = samples() - 1;
+    for( int i = 1, j = std::max( 2, samples() / 10 ); i < j; ++i )
+      {
+      if( data[i] < lth ) { lth = data[i]; begin = i; }
+      if( data[samples()-1-i] < rth )
+        { rth = data[samples()-1-i]; end = samples() - 1 - i; }
+      }
+    if( lth >= th || rth >= th ) return _istip;
+    th = std::max( lth, rth );
     int status = 0;
-    for( int i = 2; i < samples() - 2; ++i )
+    for( int i = begin + 1; i < end; ++i )
       switch( status )
         {
         case 0: if( data[i] > th + 1 ) status = 1; break;
