@@ -127,6 +127,12 @@ bool Rectangle::strictly_includes( const Rectangle & r ) const throw()
   }
 
 
+bool Rectangle::strictly_includes( int row, int col ) const throw()
+  {
+  return ( _left < col && _right > col && _top < row && _bottom > row );
+  }
+
+
 bool Rectangle::includes_hcenter( const Rectangle & r ) const throw()
   { return ( _left <= r.hcenter() && _right >= r.hcenter() ); }
 
@@ -159,6 +165,34 @@ bool Rectangle::v_overlaps( const Rectangle & r ) const throw()
   { return ( _top <= r._bottom && _bottom >= r._top ); }
 
 
+bool Rectangle::is_hcentered_in( const Rectangle & r ) const throw()
+  {
+  if( this->h_includes( r.hcenter() ) ) return true;
+  int w = std::min( r.height(), r.width() ) / 2;
+  if( width() < w )
+    {
+    int d = ( w + 1 ) / 2;
+    if( hcenter() - d <= r.hcenter() && hcenter() + d >= r.hcenter() )
+      return true;
+    }
+  return false;
+  }
+
+
+bool Rectangle::is_vcentered_in( const Rectangle & r ) const throw()
+  {
+  if( this->v_includes( r.vcenter() ) ) return true;
+  int h = std::min( r.height(), r.width() ) / 2;
+  if( height() < h )
+    {
+    int d = ( h + 1 ) / 2;
+    if( vcenter() - d <= r.vcenter() && vcenter() + d >= r.vcenter() )
+      return true;
+    }
+  return false;
+  }
+
+
 bool Rectangle::h_precedes( const Rectangle & r ) const throw()
   { return ( hcenter() < r.hcenter() ); }
 
