@@ -20,11 +20,6 @@ class Textblock : public Rectangle
   {
   mutable std::vector< Textline > data;
 
-  void add_line() throw() { data.push_back( Textline() ); }
-  void insert_line( int i ) throw();
-  void delete_line( int i ) throw();
-  void join_characters() throw();
-
 public:
   Textblock( const Rectangle & r, const Charset & charset,
              const std::vector< std::vector< Block > > & block_matrix ) throw();
@@ -36,4 +31,6 @@ public:
   void print( const Control & control ) const throw();
   void dprint( const Control & control, bool graph, bool recursive ) const throw();
   void xprint( const Control & control ) const throw();
+  void cmark( Bitmap & bitmap ) const throw();
+  void lmark( Bitmap & bitmap ) const throw();
   };
