@@ -24,48 +24,39 @@
 #include "vrhomboid.h"
 #include "track.h"
 #include "ucs.h"
+#include "bitmap.h"
 #include "block.h"
 #include "character.h"
 #include "textline.h"
 
 
-int Textline::find_space( int begin ) const throw()
+Textline::Textline( const Textline & tl ) throw()
+  : Track( tl ), data( tl.data )
   {
-  int end = begin;
-  for( ; end < characters(); ++end )
-    {
-    const Character & c = character( end );
-    if( c.guesses() && UCS::isspace( c.guess(0).code ) ) break;
-    }
-  return end;
-  }
-
-
-Textline::Textline( const Textline & l ) throw()
-  : data( l.data ), track( l.track )
-  {
-  if( l._big_initial ) _big_initial = new Character( *(l._big_initial) );
+  if( tl._big_initial ) _big_initial = new Character( *(tl._big_initial) );
   else _big_initial = 0;
   }
 
 
-Textline & Textline::operator=( const Textline & l ) throw()
+Textline & Textline::operator=( const Textline & tl ) throw()
   {
-  if( this != &l )
+  if( this != &tl )
     {
+    Track::operator=( tl );
     delete_big_initial();
-    if( l._big_initial ) _big_initial = new Character( *(l._big_initial) );
-    data = l.data; track = l.track;
+    if( tl._big_initial ) _big_initial = new Character( *(tl._big_initial) );
+    data = tl.data;
     }
   return *this;
   }
 
 
-void Textline::initialize_track() throw()
+void Textline::set_track() throw()
   {
   std::vector< Rectangle > rv;
-  for( unsigned int i = 0; i < data.size(); ++i ) rv.push_back( data[i] );
-  track.initialize( rv );
+  for( unsigned int i = 0; i < data.size(); ++i )
+    if( !data[i].maybe(' ') ) rv.push_back( data[i] );
+  Track::set_track( rv );
   }
 
 
@@ -100,19 +91,22 @@ int Textline::shift_character( const Character & c ) throw()
   }
 
 
-bool Textline::insert_space( int i ) throw()
+bool Textline::insert_space( int i, bool tab ) throw()
   {
   if( i <= 0 || i >= characters() )
     Ocrad::internal_error( "insert_space, index out of bounds" );
+  if( height() == 0 )
+    Ocrad::internal_error( "insert_space, track not set yet" );
   Character & c1 = data[i-1];
   Character & c2 = data[i];
-  if( c2.left() - c1.right() <= 1 ) return false;
-  int left = c1.right() + 1;
-  int top = ( c1.top() + c2.top() ) / 2;
-  int right = c2.left() - 1;
-  int bottom = ( c1.bottom() + c2.bottom() ) / 2;
-  Character c( Rectangle( left, top, right, bottom ), *c1.blockmap() );
-  c.add_guess( ' ', 0 );
+  int l = c1.right() + 1;
+  int r = c2.left() - 1;
+  if( l > r ) return false;
+  int t = top( ( l + r ) / 2 );
+  int b = bottom( ( l + r ) / 2 );
+  Character c( Rectangle( l, t, r, b ), *c1.blockmap() );
+  c.add_guess( ' ', tab ? 1 : 0 );
+  if( tab ) c.add_guess( '\t', 0 );
   data.insert( data.begin() + i, c );
   return true;
   }
@@ -128,33 +122,37 @@ Character & Textline::character( int i ) const throw()
 
 int Textline::mean_height() const throw()
   {
-  int sum = 0;
+  int c = 0, sum = 0;
 
-  for( int i = 0; i < characters(); ++i ) sum += data[i].height();
-  if( characters() ) sum /= characters();
+  for( int i = 0; i < characters(); ++i )
+    if( !data[i].maybe(' ') ) { ++c; sum += data[i].height(); }
+  if( c ) sum /= c;
   return sum;
   }
 
 
 int Textline::mean_width() const throw()
   {
-  int sum = 0;
+  int c = 0, sum = 0;
 
-  for( int i = 0; i < characters(); ++i ) sum += data[i].width();
-  if( characters() ) sum /= characters();
+  for( int i = 0; i < characters(); ++i )
+    if( !data[i].maybe(' ') ) { ++c; sum += data[i].width(); }
+  if( c ) sum /= c;
   return sum;
   }
 
 
-int Textline::mean_gap_width() const throw()
+int Textline::mean_gap_width( int max_gap_width ) const throw()
   {
-  int sum = 0;
+  int gaps = characters() - 1, sum = 0;
 
-  if( characters() < 2 ) return sum;
-  for( int i = 1; i < characters(); ++i )
-    if( data[i].left() > data[i-1].right() )
-      sum += data[i].left() - data[i-1].right() - 1;
-  return sum / ( characters() - 1 );
+  for( int i = 0; i < gaps; ++i )
+    {
+    int gap = data[i+1].left() - data[i].right() - 1;
+    if( gap > 0 && ( !max_gap_width || gap <= max_gap_width ) ) sum += gap;
+    }
+  if( gaps > 0 ) sum /= gaps;
+  return sum;
   }
 
 
@@ -178,15 +176,15 @@ int Textline::mean_vcenter() const throw()
   }
 
 
-void Textline::join( Textline & l ) throw()
+void Textline::join( Textline & tl ) throw()
   {
-  if( l._big_initial &&
-      ( !_big_initial || l._big_initial->h_precedes( *_big_initial ) ) )
-    big_initial( *(l._big_initial) );
+  if( tl._big_initial &&
+      ( !_big_initial || tl._big_initial->h_precedes( *_big_initial ) ) )
+    big_initial( *(tl._big_initial) );
 
-  for( int i = 0; i < l.characters(); ++i ) shift_character( l.data[i] );
+  for( int i = 0; i < tl.characters(); ++i ) shift_character( tl.data[i] );
 
-  l.delete_big_initial(); l.data.clear();
+  tl.delete_big_initial(); tl.data.clear();
   }
 
 
@@ -202,14 +200,14 @@ void Textline::dprint( const Control & control, bool graph, bool recursive )
 								const throw()
   {
   if( graph || recursive )
-    std::fprintf( control.outfile, "mean_height = %d, track.segments = %d\n",
-                  mean_height(), track.segments() );
+    std::fprintf( control.outfile, "mean height = %d, track segments = %d\n",
+                  mean_height(), segments() );
 
   if( _big_initial ) _big_initial->dprint( control, graph, recursive );
   for( int i = 0; i < characters(); ++i )
     {
     if( graph || recursive )
-      std::fprintf( control.outfile, "track.vcenter = %d\n", track.vcenter( character(i).hcenter() ) );
+      std::fprintf( control.outfile, "track vcenter = %d\n", vcenter( character(i).hcenter() ) );
     character( i ).dprint( control, graph, recursive );
     }
   std::fputs( "\n", control.outfile );
@@ -224,6 +222,14 @@ void Textline::xprint( const Control & control ) const throw()
   }
 
 
+void Textline::cmark( Bitmap & bitmap ) const throw()
+  {
+  if( _big_initial ) bitmap.draw_rectangle( *_big_initial );
+  for( int i = 0; i < characters(); ++i )
+    bitmap.draw_rectangle( character(i) );
+  }
+
+
 void Textline::recognize1( const Charset & charset ) const throw()
   {
   if( _big_initial )
@@ -241,6 +247,6 @@ void Textline::recognize1( const Charset & charset ) const throw()
   for( int i = 0; i < characters(); ++i )
     {
     Character & c = character( i );
-    c.recognize1( charset, track.vcenter( c.hcenter() ) );
+    c.recognize1( charset, vcenter( c.hcenter() ) );
     }
   }
