@@ -32,6 +32,18 @@
 // All the code in this file is provisional and will be rewritten someday
 
 
+namespace {
+
+int find_space( const std::vector< Character > & character_vector, int i ) throw()
+  {
+  while( i < (int)character_vector.size() && !character_vector[i].maybe(' ') )
+    ++i;
+  return i;
+  }
+
+} // end namespace
+
+
 void Textline::recognize2( const Charset & charset ) throw()
   {
   if( characters() == 0 ) return;
@@ -59,12 +71,13 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // try to separate lightly merged characters
+  // FIXME try all possible separation points
   for( int i = 0; i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.width() > 20 &&
-        Ocrad::similar( c.width(), 2 * mean_width(), 25 ) &&
-        Ocrad::similar( c.height(), mean_height(), 30 ) )
+    if( !c.guesses() && c.width() > 20 && c.width() > height() &&
+        4 * c.width() >= 3 * c.height() &&
+        Ocrad::similar( c.height(), mean_height(), 40 ) )
       {
       int ib = c.blocks() - 1;
       for( int k = ib - 1; k >= 0; --k )
@@ -73,19 +86,21 @@ void Textline::recognize2( const Charset & charset ) throw()
         continue;
       const Block & b = c.block( ib );
       const Blockmap & bm = *b.blockmap();
-      int cmin = 0, hmin = b.height() + 1;
-      for( int col = b.hpos(33); col <= b.hpos(66); ++col)
+      int colmin = 0, cmin = b.height() + 1;
+      for( int col = b.hpos( 30 ); col <= b.hpos( 70 ); ++col )
         {
-        int h = 0;
+        int c = 0;
         for( int row = b.top(); row <= b.bottom(); ++row )
-          if( b.compare_id( bm.id( row, col ), true ) ) ++h;
-        if( h < hmin ) { hmin = h; cmin = col; }
+          if( b.compare_id( bm.id( row, col ), true ) ) ++c;
+        if( c < cmin ) { cmin = c; colmin = col; }
         }
-      if( 5 * hmin > b.height() ) continue;
-      if( cmin <= b.left() || cmin >= b.right() ) continue;
-      Rectangle r1( b.left(), b.top(), cmin - 1, b.bottom() );
+      if( 4 * cmin > b.height() ||
+          ( 5 * cmin > b.height() &&
+            ( colmin <= b.hpos( 40 ) || colmin >= b.hpos( 60 ) ) ) ) continue;
+      if( colmin <= b.left() || colmin >= b.right() ) continue;
+      Rectangle r1( b.left(), b.top(), colmin - 1, b.bottom() );
       Block b1( r1, bm, b.id() ); b1.adjust_height();
-      Rectangle r2( cmin + 1, b.top(), b.right(), b.bottom() );
+      Rectangle r2( colmin + 1, b.top(), b.right(), b.bottom() );
       Block b2( r2, bm, b.id() ); b2.adjust_height();
       for( int j = 0; j < b.blocks(); ++j )
         {
@@ -103,8 +118,38 @@ void Textline::recognize2( const Charset & charset ) throw()
         }
       c1.recognize1( charset, c.vcenter() );
       c2.recognize1( charset, c.vcenter() );
-      if( c1.guesses() || c2.guesses() )
-        { c = c1; shift_character( c2 ); ++i; }
+      if( ( c1.guesses() && c2.guesses() ) ||
+          ( ( c1.guesses() || c2.guesses() ) && c.width() > c.height() ) )
+        {
+        c = c1; shift_character( c2 );
+        if( !c1.guesses() ) --i; else if( c2.guesses() ) ++i;
+        }
+      }
+    }
+
+  // try to recognize 1 block unrecognized characters with holes by
+  // removing small holes (noise)
+  for( int i = 0; i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( !c.guesses() && c.blocks() == 1 && c.block( 0 ).blocks() )
+      {
+      Character c1( c );
+      Block & b = c1.block( 0 );
+      for( int j = b.blocks() - 1; j >= 0; --j )
+        if( 64 * b.block( j ).size() < b.size() ) b.delete_block( j );
+      if( b.blocks() < c.block( 0 ).blocks() )
+        {
+        c1.recognize1( charset, c1.vcenter() );
+        if( c1.guesses() ) { c = c1; continue; }
+        }
+      if( b.blocks() == 1 && 16 * b.block( 0 ).size() < b.size() &&
+          Ocrad::similar( b.height(), b.width(), 40 ) )
+        {
+        b.delete_block( 0 );
+        c1.recognize1( charset, c1.vcenter() );
+        if( c1.guesses() ) { c = c1; continue; }
+        }
       }
     }
 
@@ -135,10 +180,10 @@ void Textline::recognize2( const Charset & charset ) throw()
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
+    if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
     if( c1.guesses() == 1 )
       {
       int code = c1.guess( 0 ).code;
-      if( UCS::isspace( code ) ) { begin = i + 1 ; continue; }
       if( !UCS::islower_small_ambiguous( code ) ) continue;
       if( 4 * c1.height() > 5 * mean_height() )
         { c1.only_guess( std::toupper( code ), 0 ); continue; }
@@ -148,8 +193,8 @@ void Textline::recognize2( const Charset & charset ) throw()
         {
         Character & c2 = character( j );
         if( !c2.guesses() ) continue;
+        if( c2.maybe(' ') ) break;
         int code2 = c2.guess( 0 ).code;
-        if( UCS::isspace( code2 ) ) break;
         if( code2 >= 128 || !std::isalpha( code2 ) ) continue;
         if( !capital )
           {
@@ -194,8 +239,8 @@ void Textline::recognize2( const Charset & charset ) throw()
     Character & c1 = character( i );
     if( c1.guesses() >= 1 )
       {
+      if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
       int code = c1.guess( 0 ).code;
-      if( UCS::isspace( code ) ) { begin = i + 1 ; continue; }
       if( code < 128 || c1.blocks() < 2 ) continue;
       int codeb = UCS::base_letter( code );
       if( codeb != 'o' && codeb != 'u' ) continue;
@@ -205,10 +250,10 @@ void Textline::recognize2( const Charset & charset ) throw()
         Character & c2 = character( j );
         if( c2.guesses() >= 1 )
           {
+          if( c2.maybe(' ') ) break;
           int code2 = c2.guess( 0 ).code;
           int code2b = UCS::base_letter( code2 );
           if( !code2b && code2 >= 128 ) continue;
-          if( std::isspace( code2 ) ) break;
           if( ( std::isalpha( code2 ) && 4 * b1.height() > 5 * c2.height() ) ||
               ( std::isupper( code2 ) && Ocrad::similar( b1.height(), c2.height(), 10 ) ) ||
               ( std::isalpha( code2b ) && 4 * c1.height() > 5 * c2.height() ) ||
@@ -223,18 +268,18 @@ void Textline::recognize2( const Charset & charset ) throw()
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
+    if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
     if( c1.guesses() >= 1 )
       {
       int code = c1.guess( 0 ).code;
-      if( UCS::isspace( code ) ) { begin = i + 1 ; continue; }
       if( code != 'o' && code != 'O' && code != 'l' ) continue;
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
+        if( c2.maybe(' ') ) break;
         if( c2.guesses() >= 1 )
           {
           int code2 = c2.guess( 0 ).code;
-          if( UCS::isspace( code2 ) ) break;
           if( code2 < 128 && std::isdigit( code2 ) )
             {
             if( Ocrad::similar( c1.height(), c2.height(), 10 ) )
@@ -325,7 +370,7 @@ void Textline::recognize2( const Charset & charset ) throw()
   // transform words like 'lO.OOO' into numbers like '10.000'
   for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
     {
-    end = find_space( begin );
+    end = find_space( data, begin );
     if( end - begin < 2 ) continue;
     Character & c1 = character( begin );
     if( !c1.guesses() ) continue;
@@ -369,8 +414,11 @@ void Textline::recognize2( const Charset & charset ) throw()
     if( code == '|' || code == 'l' )
       {
       int lcode = 0, rcode = 0;
-      if( i > 0 && character( i - 1 ).guesses() )
-        lcode = character( i - 1 ).guess( 0 ).code;
+      if( i > 0 )
+        { if( character( i - 1 ).guesses() )
+            lcode = character( i - 1 ).guess( 0 ).code; }
+      else if( _big_initial && _big_initial->guesses() )
+        lcode = _big_initial->guess( 0 ).code;
       if( i < characters() - 1 && character( i + 1 ).guesses() )
         rcode = character( i + 1 ).guess( 0 ).code;
       if( UCS::isupper( rcode ) &&
@@ -396,10 +444,10 @@ void Textline::recognize2( const Charset & charset ) throw()
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
+    if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
     if( c1.guesses() )
       {
       int code = c1.guess( 0 ).code;
-      if( UCS::isspace( code ) ) { begin = i + 1 ; continue; }
       if( code != 'l' && code != '|' ) continue;
       if( 4 * c1.height() > 5 * mean_height() ) continue;
       if( 5 * c1.height() < 4 * mean_height() )
@@ -408,9 +456,9 @@ void Textline::recognize2( const Charset & charset ) throw()
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
+        if( c2.maybe(' ') ) break;
         if( !c2.guesses() ) continue;
         int code2 = c2.guess( 0 ).code;
-        if( UCS::isspace( code2 ) ) break;
         if( code2 >= 128 || !std::isalpha( code2 ) ) continue;
         if( !capital )
           {
@@ -467,10 +515,10 @@ void Textline::recognize2( const Charset & charset ) throw()
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c = character( i );
+    if( c.maybe(' ') ) { begin = i + 1 ; continue; }
     if( c.guesses() )
       {
       int code = c.guess( 0 ).code;
-      if( UCS::isspace( code ) ) { begin = i + 1 ; continue; }
       if( i == begin && code == 'a' && c.guesses() == 2 &&
           c.guess( 1 ).code == 'Q' && 4 * c.height() > 5 * mean_height() )
         c.swap_guesses( 0, 1 );
@@ -493,20 +541,20 @@ void Textline::recognize2( const Charset & charset ) throw()
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
+    if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
     if( c1.guesses() )
       {
       int code = c1.guess(0).code;
-      if( UCS::isspace( code ) ) { begin = i + 1 ; continue; }
       if( c1.guesses() != 2 || code != 'B' || c1.guess(1).code != 'a' ) continue;
       if( 4 * c1.height() > 5 * mean_height() ) continue;
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
+        if( c2.maybe(' ') ) break;
         if( c2.guesses() >= 1 )
           {
           int code2 = c2.guess(0).code;
           if( code2 >= 128 ) continue;
-          if( std::isspace( code2 ) ) break;
           if( ( std::isalpha( code2 ) && 5 * c1.height() < 4 * c2.height() ) ||
               ( std::islower( code2 ) &&
                 ( c1.height() <= c2.height() ||
@@ -516,4 +564,21 @@ void Textline::recognize2( const Charset & charset ) throw()
         }
       }
     }
+
+  // join a 'n' followed by a 'I' into a 'm'
+  for( int i = 0; i < characters() - 1; ++i )
+    {
+    Character & c1 = character( i );
+    Character & c2 = character( i + 1 );
+    if( c1.guesses() == 1 && c2.guesses() == 1 )
+      {
+      int code1 = c1.guess( 0 ).code;
+      int code2 = c2.guess( 0 ).code;
+      if( code1 == 'n' && ( code2 == 'I' || code2 == 'l' ) &&
+          Ocrad::similar( c1.height(), c2.height(), 10 ) &&
+          c2.left() - c1.right() < c2.width() )
+        { c1.join( c2 ); c1.only_guess( 'm', 0 ); delete_character( i + 1 ); }
+      }
+    }
+
   }
