@@ -35,25 +35,56 @@
 Textpage::Textpage( const Bitmap & page_image, const char * filename,
                     const Control & control ) throw()
   : Rectangle( 0, 0, page_image.width() - 1, page_image.height() - 1 ),
-  name( filename )
+  name( filename ), bmp( 0 )
   {
-  // build a Textblock for every rectangle of text
-  for( int i = 0; i < page_image.rectangles(); ++i )
-    {
-    Blockmap * bmp = new Blockmap( page_image, i, control.debug_level );
-    if( !bmp->blocks() ) { delete bmp; continue; }
+  if( control.debug_level < 0 || control.debug_level > 100 ) return;
 
-    if( control.debug_level >= 90 && control.debug_level <= 100 )
+  bmp = new Blockmap( page_image, control.debug_level );
+
+  if( control.debug_level >= 98 )
+    {
+    if( control.outfile )
+      bmp->print( control.outfile, control.debug_level );
+    return;
+    }
+  if( control.debug_level > 89 ) return;
+
+  // build a Textblock for every zone with text on blockmap
+  for( int i = 0; i < bmp->zones(); ++i )
+    {
+    Textblock * tbp = new Textblock( bmp->zone_vector()[i].rectangle,
+                      control.charset, bmp->zone_vector()[i].block_matrix );
+    if( tbp->lines() ) tbpv.push_back( tbp );
+    else delete tbp;
+    }
+  if( control.debug_level == 0 || !control.outfile ) return;
+  if( control.debug_level >= 86 )
+    {
+    bool graph = ( control.debug_level >= 88 );
+    bool recursive = ( control.debug_level & 1 );
+    for( int i = 0; i < textblocks(); ++i )
+      tbpv[i]->dprint( control, graph, recursive );
+    return;
+    }
+  if( control.debug_level > 77 ) return;
+  if( control.debug_level >= 70 )
+    {
+    Bitmap bitmap( bmp->data_matrix() );
+    if( ( control.debug_level - 70 ) & 1 )	// mark zones
       {
-      if( control.outfile )
-        bmp->print( control.outfile, control.debug_level );
-      delete bmp;
+      for( int i = 0; i < bmp->zones(); ++i )
+        bitmap.draw_rectangle( bmp->zone_vector()[i].rectangle );
       }
-    else if( control.debug_level < 90 )
+    if( ( control.debug_level - 70 ) & 2 )	// mark lines
       {
-      bmpv.push_back( bmp );
-      tbpv.push_back( new Textblock( page_image.rectangle_vector()[i], control.charset, bmp->block_matrix() ) );
+      for( int i = 0; i < textblocks(); ++i ) tbpv[i]->lmark( bitmap );
       }
+    if( ( control.debug_level - 70 ) & 4 )	// mark characters
+      {
+      for( int i = 0; i < textblocks(); ++i ) tbpv[i]->cmark( bitmap );
+      }
+    bitmap.save( control.outfile, control.filetype );
+    return;
     }
   }
 
@@ -61,18 +92,10 @@ Textpage::Textpage( const Bitmap & page_image, const char * filename,
 Textpage::~Textpage() throw()
   {
   for( int i = textblocks() - 1; i >= 0; --i ) delete tbpv[i];
-  for( int i = blockmaps() - 1; i >= 0; --i ) delete bmpv[i];
+  if( bmp ) delete bmp;
   }
 
 /*
-const Blockmap & Textpage::blockmap( int i ) const throw()
-  {
-  if( i < 0 || i >= blockmaps() )
-    Ocrad::internal_error( "Textpage::blockmap, index out of bounds" );
-  return *(bmpv[i]);
-  }
-
-
 const Textblock & Textpage::textblock( int i ) const throw()
   {
   if( i < 0 || i >= textblocks() )
@@ -89,25 +112,17 @@ void Textpage::print( const Control & control ) const throw()
   }
 
 
-void Textpage::dprint( const Control & control, bool graph, bool recursive ) const throw()
-  {
-  if( control.outfile )
-    for( int i = 0; i < textblocks(); ++i )
-      tbpv[i]->dprint( control, graph, recursive );
-  }
-
-
 void Textpage::xprint( const Control & control ) const throw()
   {
   if( !control.exportfile ) return;
 
   std::fprintf( control.exportfile, "source file %s\n", name.c_str() );
-  std::fprintf( control.exportfile, "total blocks %d\n", textblocks() );
+  std::fprintf( control.exportfile, "total text blocks %d\n", textblocks() );
 
   for( int i = 0; i < textblocks(); ++i )
     {
     const Textblock & tb = *(tbpv[i]);
-    std::fprintf( control.exportfile, "block %d %d %d %d %d\n", i + 1,
+    std::fprintf( control.exportfile, "text block %d %d %d %d %d\n", i + 1,
                   tb.left(), tb.top(), tb.width(), tb.height() );
     tb.xprint( control );
     }
