@@ -22,7 +22,7 @@ class Textblock;
 class Textpage : public Rectangle
   {
   const std::string name;
-  std::vector< Blockmap * > bmpv;
+  const Blockmap * bmp;
   std::vector< Textblock * > tbpv;
 
 public:
@@ -30,12 +30,10 @@ public:
             const Control & control ) throw();
   ~Textpage() throw();
 
-//  const Blockmap & blockmap( int i ) const throw();
-  int blockmaps() const throw() { return bmpv.size(); }
+//  const Blockmap & blockmap() const throw() { return *bmp; }
 //  const Textblock & textblock( int i ) const throw();
   int textblocks() const throw() { return tbpv.size(); }
 
   void print( const Control & control ) const throw();
-  void dprint( const Control & control, bool graph, bool recursive ) const throw();
   void xprint( const Control & control ) const throw();
   };
