@@ -22,13 +22,20 @@ class Track		// vector of Vrhomboids tracking a Textline.
 
 public:
   Track() throw() {}
+  Track( const Track & t ) throw() : data( t.data ) {}
+  Track & operator=( const Track & t ) throw()
+    { if( this != &t ) { data = t.data; } return *this; }
 
-  void initialize( const std::vector< Rectangle > rectangle_vector ) throw();
+  void set_track( const std::vector< Rectangle > & rectangle_vector ) throw();
 
   int segments() const throw() { return data.size(); }
-  int height() const throw();
+  int height() const throw() { return data.size() ? data[0].height() : 0; }
+  int left() const throw() { return data.size() ? data[0].left() : 0; }
+  int right() const throw() { return data.size() ? data.back().right() : 0; }
 
   int vcenter( int col ) const throw();
   int bottom( int col ) const throw();
   int top( int col ) const throw();
+
+  static int space_width_limit( int mean_width ) throw() { return 3 * mean_width; }
   };
